import 'dart:async';

enum PasswordStrength {
  tiny,
  veryWeek,
  week,
  good,
  strong,
}

enum PasswordImprovementTips {
  addLowercaseChars,
  addUppercaseChars,
  addNumbers,
  addSymbols
}

class PasswordStrengthVerificator {
  final int minPasswordLength;

  const PasswordStrengthVerificator({required this.minPasswordLength});

  FutureOr<PasswordStrength> call(FutureOr<String> password) async {
    PasswordStrength response = PasswordStrength.veryWeek;
    final realPassword = (await password).trim();
    if (realPassword.length < minPasswordLength) {
      response = PasswordStrength.tiny;
    } else {
      int points = 0;
      if (_lowerCase.hasMatch(realPassword)) {
        points += 1;
      }

      if (_upperCase.hasMatch(realPassword)) {
        points += 1;
      }

      if (_numberCase.hasMatch(realPassword)) {
        points += 1;
      }

      if (_symbolsCase.hasMatch(realPassword)) {
        points += 1;
      }

      switch (points) {
        case 0:
          response = PasswordStrength.veryWeek;
          break;
        case 1:
          response = PasswordStrength.veryWeek;
          break;
        case 2:
          response = PasswordStrength.week;
          break;
        case 3:
          response = PasswordStrength.good;
          break;
        case 4:
          response = PasswordStrength.strong;
          break;
      }
    }

    return response;
  }

  FutureOr<Set<PasswordImprovementTips>> getPasswordImprovementTips(
      FutureOr<String> password) async {
    final realPassword = (await password).trim();
    final response = <PasswordImprovementTips>{};
    if (!_lowerCase.hasMatch(realPassword)) {
      response.add(PasswordImprovementTips.addLowercaseChars);
    }

    if (!_upperCase.hasMatch(realPassword)) {
      response.add(PasswordImprovementTips.addUppercaseChars);
    }

    if (!_numberCase.hasMatch(realPassword)) {
      response.add(PasswordImprovementTips.addNumbers);
    }

    if (!_symbolsCase.hasMatch(realPassword)) {
      response.add(PasswordImprovementTips.addSymbols);
    }

    return response;
    
  }

  RegExp get _lowerCase => RegExp(
        r'^(?=.*?[a-z]).{'
        '$minPasswordLength'
        r',}$',
      );

  RegExp get _upperCase => RegExp(
        r'^(?=.*?[A-Z]).{'
        '$minPasswordLength'
        r',}$',
      );

  RegExp get _numberCase => RegExp(
        r'^(?=.*?[0-9]).{'
        '$minPasswordLength'
        r',}$',
      );

  RegExp get _symbolsCase => RegExp(
        r'^(?=.*?[!@#\$&*~]).{'
        '$minPasswordLength'
        r',}$',
      );
}
